@@ -1,4 +1,4 @@
-# Copyright 2021 The DDSP Authors.
+# Copyright 2022 The DDSP Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 """Library of FFT operations for loss functions and conditioning."""
 
 import crepe
+from ddsp import core
 from ddsp.core import safe_log
 from ddsp.core import tf_float32
 import gin
@@ -25,10 +26,10 @@ import tensorflow.compat.v2 as tf
 import tensorflow_probability as tfp
 
 CREPE_SAMPLE_RATE = 16000
-_CREPE_FRAME_SIZE = 1024
+CREPE_FRAME_SIZE = 1024
 
-F0_RANGE = 127.0  # MIDI
-LD_RANGE = 120.0  # dB
+F0_RANGE = 127.0  # MIDI.
+DB_RANGE = core.DB_RANGE  # dB (80.0).
 
 
 def stft(audio, frame_size=2048, overlap=0.75, pad_end=True):
@@ -56,18 +57,11 @@ def stft_np(audio, frame_size=2048, overlap=0.75, pad_end=True):
   is_2d = (len(audio.shape) == 2)
 
   if pad_end:
-    n_samples_initial = int(audio.shape[-1])
-    n_frames = int(np.ceil(n_samples_initial / hop_size))
-    n_samples_final = (n_frames - 1) * hop_size + frame_size
-    pad = n_samples_final - n_samples_initial
-    padding = ((0, 0), (0, pad)) if is_2d else ((0, pad),)
-    audio = np.pad(audio, padding, 'constant')
+    audio = pad(audio, frame_size, hop_size, 'same', axis=is_2d).numpy()
 
   def stft_fn(y):
-    return librosa.stft(y=y,
-                        n_fft=int(frame_size),
-                        hop_length=hop_size,
-                        center=False).T
+    return librosa.stft(
+        y=y, n_fft=int(frame_size), hop_length=hop_size, center=False).T
 
   s = np.stack([stft_fn(a) for a in audio]) if is_2d else stft_fn(audio)
   return s
@@ -142,154 +136,218 @@ def compute_mfcc(audio,
   return mfccs[..., :mfcc_bins]
 
 
-def diff(x, axis=-1):
-  """Take the finite difference of a tensor along an axis.
+def get_framed_lengths(input_length, frame_size, hop_size, padding='center'):
+  """Give a strided framing, such as tf.signal.frame, gives output lengths.
 
   Args:
-    x: Input tensor of any dimension.
-    axis: Axis on which to take the finite difference.
+    input_length: Original length along the dimension to be framed.
+    frame_size: Size of frames for striding.
+    hop_size: Striding, space between frames.
+    padding: Type of padding to apply, ['valid', 'same', 'center']. 'valid' is
+      a no-op. 'same' applies padding to the end such that
+      n_frames = n_t / hop_size. 'center' applies padding to both ends such that
+      each frame timestamp is centered and n_frames = n_t / hop_size + 1.
 
   Returns:
-    d: Tensor with size less than x by 1 along the difference dimension.
-
-  Raises:
-    ValueError: Axis out of range for tensor.
+    n_frames: Number of frames left after striding.
+    padded_length: Length of the padded signal before striding.
   """
-  shape = x.shape.as_list()
-  ndim = len(shape)
-  if axis >= ndim:
-    raise ValueError('Invalid axis index: %d for tensor with only %d axes.' %
-                     (axis, ndim))
+  # Use numpy since this function isn't used dynamically.
+  def get_n_frames(length):
+    return int(np.floor((length - frame_size) / hop_size)) + 1
 
-  begin_back = [0 for _ in range(ndim)]
-  begin_front = [0 for _ in range(ndim)]
-  begin_front[axis] = 1
+  if padding == 'valid':
+    padded_length = input_length
+    n_frames = get_n_frames(input_length)
 
-  shape[axis] -= 1
-  slice_front = tf.slice(x, begin_front, shape)
-  slice_back = tf.slice(x, begin_back, shape)
-  d = slice_front - slice_back
-  return d
+  elif padding == 'center':
+    padded_length = input_length + frame_size
+    n_frames = get_n_frames(padded_length)
+
+  elif padding == 'same':
+    n_frames = int(np.ceil(input_length / hop_size))
+    padded_length = (n_frames - 1) * hop_size + frame_size
+
+  return n_frames, padded_length
 
 
-def amplitude_to_db(amplitude, use_tf=False):
-  """Converts amplitude to decibels."""
-  lib = tf if use_tf else np
-  log10 = (lambda x: tf.math.log(x) / tf.math.log(10.0)) if use_tf else np.log10
-  amin = 1e-20  # Avoid log(0) instabilities.
-  db = log10(lib.maximum(amin, amplitude))
-  db *= 20.0
-  return db
+def pad(x, frame_size, hop_size, padding='center',
+        axis=1, mode='CONSTANT', constant_values=0):
+  """Pad a tensor for strided framing such as tf.signal.frame.
+
+  Args:
+    x: Tensor to pad, any shape.
+    frame_size: Size of frames for striding.
+    hop_size: Striding, space between frames.
+    padding: Type of padding to apply, ['valid', 'same', 'center']. 'valid' is
+      a no-op. 'same' applies padding to the end such that
+      n_frames = n_t / hop_size. 'center' applies padding to both ends such that
+      each frame timestamp is centered and n_frames = n_t / hop_size + 1.
+    axis: Axis along which to pad `x`.
+    mode: Padding mode for tf.pad(). One of "CONSTANT", "REFLECT", or
+      "SYMMETRIC" (case-insensitive).
+    constant_values: Passthrough kwarg for tf.pad().
+
+  Returns:
+    A padded version of `x` along axis. Output sizes can be computed separately
+      with strided_lengths.
+  """
+  x = tf_float32(x)
+
+  if padding == 'valid':
+    return x
+
+  if hop_size > frame_size:
+    raise ValueError(f'During padding, frame_size ({frame_size})'
+                     f' must be greater than hop_size ({hop_size}).')
+
+  if len(x.shape) <= 1:
+    axis = 0
+
+  n_t = x.shape[axis]
+  _, n_t_padded = get_framed_lengths(n_t, frame_size, hop_size, padding)
+  pads = [[0, 0] for _ in range(len(x.shape))]
+
+  if padding == 'same':
+    pad_amount = int(n_t_padded - n_t)
+    pads[axis] = [0, pad_amount]
+
+  elif padding == 'center':
+    pad_amount = int(frame_size // 2)  # Symmetric even padding like librosa.
+    pads[axis] = [pad_amount, pad_amount]
+
+  else:
+    raise ValueError('`padding` must be one of [\'center\', \'same\''
+                     f'\'valid\'], received ({padding}).')
+
+  return tf.pad(x, pads, mode=mode, constant_values=constant_values)
 
 
-def db_to_amplitude(db):
-  """Converts decibels to amplitude."""
-  return 10.0**(db / 20.0)
+def compute_rms_energy(audio,
+                       sample_rate=16000,
+                       frame_rate=250,
+                       frame_size=512,
+                       padding='center'):
+  """Compute root mean squared energy of audio."""
+  audio = tf_float32(audio)
+  hop_size = sample_rate // frame_rate
+  audio = pad(audio, frame_size, hop_size, padding=padding)
+  audio_frames = tf.signal.frame(audio, frame_size, hop_size, pad_end=False)
+  rms_energy = tf.reduce_mean(audio_frames**2.0, axis=-1)**0.5
+  return rms_energy
+
+
+def compute_power(audio,
+                  sample_rate=16000,
+                  frame_rate=250,
+                  frame_size=512,
+                  ref_db=0.0,
+                  range_db=DB_RANGE,
+                  padding='center'):
+  """Compute power of audio in dB."""
+  rms_energy = compute_rms_energy(
+      audio, sample_rate, frame_rate, frame_size, padding=padding)
+  power_db = core.amplitude_to_db(
+      rms_energy, ref_db=ref_db, range_db=range_db, use_tf=True)
+  return power_db
 
 
 @gin.register
 def compute_loudness(audio,
                      sample_rate=16000,
                      frame_rate=250,
-                     n_fft=2048,
-                     range_db=LD_RANGE,
-                     ref_db=20.7,
-                     use_tf=False,
-                     pad_end=True):
-  """Perceptual loudness in dB, relative to white noise, amplitude=1.
+                     n_fft=512,
+                     range_db=DB_RANGE,
+                     ref_db=0.0,
+                     use_tf=True,
+                     padding='center'):
+  """Perceptual loudness (weighted power) in dB.
 
   Function is differentiable if use_tf=True.
   Args:
     audio: Numpy ndarray or tensor. Shape [batch_size, audio_length] or
-      [batch_size,].
+      [audio_length,].
     sample_rate: Audio sample rate in Hz.
     frame_rate: Rate of loudness frames in Hz.
     n_fft: Fft window size.
     range_db: Sets the dynamic range of loudness in decibles. The minimum
       loudness (per a frequency bin) corresponds to -range_db.
     ref_db: Sets the reference maximum perceptual loudness as given by
-      (A_weighting + 10 * log10(abs(stft(audio))**2.0). The default value
-      corresponds to white noise with amplitude=1.0 and n_fft=2048. There is a
-      slight dependence on fft_size due to different granularity of perceptual
-      weighting.
+      (A_weighting + 10 * log10(abs(stft(audio))**2.0). The old (<v2.0.0)
+      default value corresponded to white noise with amplitude=1.0 and
+      n_fft=2048. With v2.0.0 it was set to 0.0 to be more consistent with power
+      calculations that have a natural scale for 0 dB being amplitude=1.0.
     use_tf: Make function differentiable by using tensorflow.
-    pad_end: Add zero padding at end of audio (like `same` convolution).
+    padding: 'same', 'valid', or 'center'.
 
   Returns:
     Loudness in decibels. Shape [batch_size, n_frames] or [n_frames,].
   """
-  if sample_rate % frame_rate != 0:
-    raise ValueError(
-        'frame_rate: {} must evenly divide sample_rate: {}.'
-        'For default frame_rate: 250Hz, suggested sample_rate: 16kHz or 48kHz'
-        .format(frame_rate, sample_rate))
-
   # Pick tensorflow or numpy.
   lib = tf if use_tf else np
+  reduce_mean = tf.reduce_mean if use_tf else np.mean
+  stft_fn = stft if use_tf else stft_np
 
   # Make inputs tensors for tensorflow.
-  audio = tf_float32(audio) if use_tf else audio
+  frame_size = n_fft
+  hop_size = sample_rate // frame_rate
+  audio = pad(audio, frame_size, hop_size, padding=padding)
+  audio = audio if use_tf else np.array(audio)
 
   # Temporarily a batch dimension for single examples.
   is_1d = (len(audio.shape) == 1)
   audio = audio[lib.newaxis, :] if is_1d else audio
 
   # Take STFT.
-  hop_size = sample_rate // frame_rate
-  overlap = 1 - hop_size / n_fft
-  stft_fn = stft if use_tf else stft_np
-  s = stft_fn(audio, frame_size=n_fft, overlap=overlap, pad_end=pad_end)
+  overlap = 1 - hop_size / frame_size
+  s = stft_fn(audio, frame_size=frame_size, overlap=overlap, pad_end=False)
 
   # Compute power.
   amplitude = lib.abs(s)
-  power_db = amplitude_to_db(amplitude, use_tf=use_tf)
+  power = amplitude**2
 
   # Perceptual weighting.
   frequencies = librosa.fft_frequencies(sr=sample_rate, n_fft=n_fft)
   a_weighting = librosa.A_weighting(frequencies)[lib.newaxis, lib.newaxis, :]
-  loudness = power_db + a_weighting
 
-  # Set dynamic range.
-  loudness -= ref_db
-  loudness = lib.maximum(loudness, -range_db)
-  mean = tf.reduce_mean if use_tf else np.mean
+  # Perform weighting in linear scale, a_weighting given in decibels.
+  weighting = 10**(a_weighting/10)
+  power = power * weighting
 
-  # Average over frequency bins.
-  loudness = mean(loudness, axis=-1)
+  # Average over frequencies (weighted power per a bin).
+  avg_power = reduce_mean(power, axis=-1)
+  loudness = core.power_to_db(avg_power,
+                              ref_db=ref_db,
+                              range_db=range_db,
+                              use_tf=use_tf)
 
   # Remove temporary batch dimension.
   loudness = loudness[0] if is_1d else loudness
 
-  # Compute expected length of loudness vector
-  n_secs = audio.shape[-1] / float(
-      sample_rate)  # `n_secs` can have milliseconds
-  expected_len = int(n_secs * frame_rate)
-
-  # Pad with `-range_db` noise floor or trim vector
-  loudness = pad_or_trim_to_expected_length(
-      loudness, expected_len, -range_db, use_tf=use_tf)
   return loudness
 
 
 @gin.register
-def compute_f0(audio, sample_rate, frame_rate, viterbi=True):
+def compute_f0(audio, frame_rate, viterbi=True, padding='center'):
   """Fundamental frequency (f0) estimate using CREPE.
 
   This function is non-differentiable and takes input as a numpy array.
   Args:
-    audio: Numpy ndarray of single audio example. Shape [audio_length,].
-    sample_rate: Sample rate in Hz.
+    audio: Numpy ndarray of single audio (16kHz) example. Shape [audio_length,].
     frame_rate: Rate of f0 frames in Hz.
     viterbi: Use Viterbi decoding to estimate f0.
+    padding: Apply zero-padding for centered frames.
+      'same', 'valid', or 'center'.
 
   Returns:
     f0_hz: Fundamental frequency in Hz. Shape [n_frames,].
     f0_confidence: Confidence in Hz estimate (scaled [0, 1]). Shape [n_frames,].
   """
-
-  n_secs = len(audio) / float(sample_rate)  # `n_secs` can have milliseconds
+  sample_rate = CREPE_SAMPLE_RATE
   crepe_step_size = 1000 / frame_rate  # milliseconds
-  expected_len = int(n_secs * frame_rate)
+  hop_size = sample_rate // frame_rate
+
+  audio = pad(audio, CREPE_FRAME_SIZE, hop_size, padding)
   audio = np.asarray(audio)
 
   # Compute f0 with crepe.
@@ -301,54 +359,14 @@ def compute_f0(audio, sample_rate, frame_rate, viterbi=True):
       center=False,
       verbose=0)
 
-  # Postprocessing on f0_hz
-  f0_hz = pad_or_trim_to_expected_length(f0_hz, expected_len, 0)  # pad with 0
+  # Postprocessing.
   f0_hz = f0_hz.astype(np.float32)
-
-  # Postprocessing on f0_confidence
-  f0_confidence = pad_or_trim_to_expected_length(f0_confidence, expected_len, 1)
-  f0_confidence = np.nan_to_num(f0_confidence)   # Set nans to 0 in confidence
   f0_confidence = f0_confidence.astype(np.float32)
+  f0_confidence = np.nan_to_num(f0_confidence)   # Set nans to 0 in confidence
+
   return f0_hz, f0_confidence
 
 
-def compute_rms_energy(audio,
-                       sample_rate=16000,
-                       frame_rate=250,
-                       frame_size=2048,
-                       pad_end=True):
-  """Compute root mean squared energy of audio."""
-  audio = tf_float32(audio)
-  hop_size = sample_rate // frame_rate
-  audio_frames = tf.signal.frame(audio, frame_size, hop_size, pad_end=pad_end)
-  rms_energy = tf.reduce_mean(audio_frames**2.0, axis=-1)**0.5
-  if pad_end:
-    n_samples = audio.shape[0] if len(audio.shape) == 1 else audio.shape[1]
-    n_secs = n_samples / float(sample_rate)  # `n_secs` can have milliseconds
-    expected_len = int(n_secs * frame_rate)
-    return pad_or_trim_to_expected_length(rms_energy, expected_len, use_tf=True)
-  else:
-    return rms_energy
-
-
-def compute_power(audio,
-                  sample_rate=16000,
-                  frame_rate=250,
-                  frame_size=1024,
-                  range_db=LD_RANGE,
-                  ref_db=20.7,
-                  pad_end=True):
-  """Compute power of audio in dB."""
-  # TODO(hanoih@): enable `use_tf` to be True or False like `compute_loudness`
-  rms_energy = compute_rms_energy(
-      audio, sample_rate, frame_rate, frame_size, pad_end)
-  power_db = amplitude_to_db(rms_energy**2, use_tf=True)
-  # Set dynamic range.
-  power_db -= ref_db
-  power_db = tf.maximum(power_db, -range_db)
-  return power_db
-
-
 def pad_or_trim_to_expected_length(vector,
                                    expected_len,
                                    pad_value=0,
@@ -485,10 +503,12 @@ class PretrainedCREPE(tf.keras.Model):
     frames /= std[:, None]
     return frames
 
-  def predict_f0_and_confidence(self, audio, viterbi=False):
+  def predict_f0_and_confidence(self, audio, viterbi=False, padding='center'):
     audio = audio[None, :] if len(audio.shape) == 1 else audio
     batch_size = audio.shape[0]
 
+    audio = pad(audio, self.frame_size, self.hop_size, padding=padding)
+
     frames = self.batch_frames(audio)
     frames = self.normalize_frames(frames)
     acts = self.core_model(frames, training=False)
