@@ -1,4 +1,4 @@
-# Copyright 2021 The DDSP Authors.
+# Copyright 2022 The DDSP Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,7 +24,7 @@ import tensorflow.compat.v2 as tf
 
 
 
-def _load_audio_as_array(audio_path: str, sample_rate: int) -> np.array:
+def _load_audio_as_array(audio_path, sample_rate):
   """Load audio file at specified sample rate and return an array.
 
   When `sample_rate` > original SR of audio file, Pydub may miss samples when
@@ -61,22 +61,24 @@ def _load_audio(audio_path, sample_rate):
   return {'audio': audio}
 
 
-def add_loudness(ex, sample_rate, frame_rate, n_fft=2048):
-  """Add loudness in dB."""
-  beam.metrics.Metrics.counter('prepare-tfrecord', 'compute-loudness').inc()
+def _chunk_audio(ex, sample_rate, chunk_secs):
+  """Pad audio and split into chunks."""
+  beam.metrics.Metrics.counter('prepare-tfrecord', 'load-audio').inc()
   audio = ex['audio']
-  mean_loudness_db = spectral_ops.compute_loudness(audio, sample_rate,
-                                                   frame_rate, n_fft)
-  ex = dict(ex)
-  ex['loudness_db'] = mean_loudness_db.astype(np.float32)
-  return ex
+  chunk_size = int(chunk_secs * sample_rate)
+  chunks = tf.signal.frame(audio, chunk_size, chunk_size, pad_end=True)
+  n_chunks = chunks.shape[0]
+  for i in range(n_chunks):
+    yield {'audio': chunks[i]}
 
 
-def _add_f0_estimate(ex, sample_rate, frame_rate):
+def _add_f0_estimate(ex, frame_rate, center, viterbi):
   """Add fundamental frequency (f0) estimate using CREPE."""
   beam.metrics.Metrics.counter('prepare-tfrecord', 'estimate-f0').inc()
   audio = ex['audio']
-  f0_hz, f0_confidence = spectral_ops.compute_f0(audio, sample_rate, frame_rate)
+  padding = 'center' if center else 'same'
+  f0_hz, f0_confidence = spectral_ops.compute_f0(
+      audio, frame_rate, viterbi=viterbi, padding=padding)
   ex = dict(ex)
   ex.update({
       'f0_hz': f0_hz.astype(np.float32),
@@ -85,24 +87,38 @@ def _add_f0_estimate(ex, sample_rate, frame_rate):
   return ex
 
 
-def split_example(ex, sample_rate, frame_rate, window_secs, hop_secs):
+def _add_loudness(ex, sample_rate, frame_rate, n_fft, center):
+  """Add loudness in dB."""
+  beam.metrics.Metrics.counter('prepare-tfrecord', 'compute-loudness').inc()
+  audio = ex['audio']
+  padding = 'center' if center else 'same'
+  loudness_db = spectral_ops.compute_loudness(
+      audio, sample_rate, frame_rate, n_fft, padding=padding)
+  ex = dict(ex)
+  ex['loudness_db'] = loudness_db.numpy().astype(np.float32)
+  return ex
+
+
+def _split_example(ex, sample_rate, frame_rate, example_secs, hop_secs, center):
   """Splits example into windows, padding final window if needed."""
 
-  def get_windows(sequence, rate):
-    window_size = int(window_secs * rate)
+  def get_windows(sequence, rate, center):
+    window_size = int(example_secs * rate)
+    if center:
+      window_size += 1
     hop_size = int(hop_secs * rate)
-    n_windows = int(np.ceil((len(sequence) - window_size) / hop_size)) + 1
-    n_samples_padded = (n_windows - 1) * hop_size + window_size
-    n_padding = n_samples_padded - len(sequence)
-    sequence = np.pad(sequence, (0, n_padding), mode='constant')
-    for window_end in range(window_size, len(sequence) + 1, hop_size):
-      yield sequence[window_end - window_size:window_end]
+    # Don't pad the end.
+    n_windows = int(np.floor((len(sequence) - window_size) / hop_size)) + 1
+    for i in range(n_windows):
+      start = i * hop_size
+      end = start + window_size
+      yield sequence[start:end]
 
   for audio, loudness_db, f0_hz, f0_confidence in zip(
-      get_windows(ex['audio'], sample_rate),
-      get_windows(ex['loudness_db'], frame_rate),
-      get_windows(ex['f0_hz'], frame_rate),
-      get_windows(ex['f0_confidence'], frame_rate)):
+      get_windows(ex['audio'], sample_rate, center=False),
+      get_windows(ex['loudness_db'], frame_rate, center),
+      get_windows(ex['f0_hz'], frame_rate, center),
+      get_windows(ex['f0_confidence'], frame_rate, center)):
     beam.metrics.Metrics.counter('prepare-tfrecord', 'split-example').inc()
     yield {
         'audio': audio,
@@ -112,7 +128,7 @@ def split_example(ex, sample_rate, frame_rate, window_secs, hop_secs):
     }
 
 
-def float_dict_to_tfexample(float_dict):
+def _float_dict_to_tfexample(float_dict):
   """Convert dictionary of float arrays to tf.train.Example proto."""
   return tf.train.Example(
       features=tf.train.Features(
@@ -122,12 +138,12 @@ def float_dict_to_tfexample(float_dict):
           }))
 
 
-def add_key(example):
+def _add_key(example):
   """Add a key to this example by taking the hash of the values."""
   return hash(example['audio'].tobytes()), example
 
 
-def eval_split_partition_fn(example, num_partitions, eval_fraction, all_ids):
+def _eval_split_partition_fn(example, num_partitions, eval_fraction, all_ids):
   """Partition function to split into train/eval based on the hash ids."""
   del num_partitions
   example_id = example[0]
@@ -143,10 +159,12 @@ def prepare_tfrecord(input_audio_paths,
                      num_shards=None,
                      sample_rate=16000,
                      frame_rate=250,
-                     window_secs=4,
+                     example_secs=4,
                      hop_secs=1,
                      eval_split_fraction=0.0,
-                     coarse_chunk_secs=20.0,
+                     chunk_secs=20.0,
+                     center=False,
+                     viterbi=True,
                      pipeline_options=''):
   """Prepares a TFRecord for use in training, evaluation, and prediction.
 
@@ -160,18 +178,43 @@ def prepare_tfrecord(input_audio_paths,
     sample_rate: The sample rate to use for the audio.
     frame_rate: The frame rate to use for f0 and loudness features. If set to
       None, these features will not be computed.
-    window_secs: The size of the sliding window (in seconds) to use to split the
-      audio and features. If 0, they will not be split.
+    example_secs: The size of the sliding window (in seconds) to use to split
+      the audio and features. If 0, they will not be split.
     hop_secs: The number of seconds to hop when computing the sliding windows.
     eval_split_fraction: Fraction of the dataset to reserve for eval split. If
       set to 0, no eval split is created.
-    coarse_chunk_secs: Chunk size in seconds used to split the input audio
-      files. This is used to split large audio files into manageable chunks
-      for better parallelization and to enable non-overlapping train/eval
-      splits.
+    chunk_secs: Chunk size in seconds used to split the input audio
+      files. This is used to split large audio files into manageable chunks for
+      better parallelization and to enable non-overlapping train/eval splits.
+    center: Provide zero-padding to audio so that frame timestamps will be
+      centered.
+    viterbi: Use viterbi decoding of pitch.
     pipeline_options: An iterable of command line arguments to be used as
       options for the Beam Pipeline.
   """
+  def postprocess_pipeline(examples, output_path, stage_name=''):
+    """After chunking, features, and train-eval split, create TFExamples."""
+    if stage_name:
+      stage_name = f'_{stage_name}'
+
+    if example_secs:
+      examples |= f'split_examples{stage_name}' >> beam.FlatMap(
+          _split_example,
+          sample_rate=sample_rate,
+          frame_rate=frame_rate,
+          example_secs=example_secs,
+          hop_secs=hop_secs,
+          center=center)
+    _ = (
+        examples
+        | f'reshuffle{stage_name}' >> beam.Reshuffle()
+        | f'make_tfexample{stage_name}' >> beam.Map(_float_dict_to_tfexample)
+        | f'write{stage_name}' >> beam.io.tfrecordio.WriteToTFRecord(
+            output_path,
+            num_shards=num_shards,
+            coder=beam.coders.ProtoCoder(tf.train.Example)))
+
+  # Start the pipeline.
   pipeline_options = beam.options.pipeline_options.PipelineOptions(
       pipeline_options)
   with beam.Pipeline(options=pipeline_options) as pipeline:
@@ -180,36 +223,32 @@ def prepare_tfrecord(input_audio_paths,
         | beam.Create(input_audio_paths)
         | beam.Map(_load_audio, sample_rate))
 
+    # Split into chunks for train/eval split and better parallelism.
+    if chunk_secs:
+      examples |= beam.FlatMap(
+          _chunk_audio,
+          sample_rate=sample_rate,
+          chunk_secs=chunk_secs)
+
+    # Add features.
     if frame_rate:
       examples = (
           examples
-          | beam.Map(_add_f0_estimate, sample_rate, frame_rate)
-          | beam.Map(add_loudness, sample_rate, frame_rate))
-
-    if coarse_chunk_secs:
-      examples |= beam.FlatMap(split_example, sample_rate, frame_rate,
-                               coarse_chunk_secs, coarse_chunk_secs)
-
-    def postprocess_pipeline(examples, output_path, stage_name=''):
-      if stage_name:
-        stage_name = f'_{stage_name}'
-
-      if window_secs:
-        examples |= f'create_batches{stage_name}' >> beam.FlatMap(
-            split_example, sample_rate, frame_rate, window_secs, hop_secs)
-      _ = (
-          examples
-          | f'reshuffle{stage_name}' >> beam.Reshuffle()
-          | f'make_tfexample{stage_name}' >> beam.Map(float_dict_to_tfexample)
-          | f'write{stage_name}' >> beam.io.tfrecordio.WriteToTFRecord(
-              output_path,
-              num_shards=num_shards,
-              coder=beam.coders.ProtoCoder(tf.train.Example)))
+          | beam.Map(_add_f0_estimate,
+                     frame_rate=frame_rate,
+                     center=center,
+                     viterbi=viterbi)
+          | beam.Map(_add_loudness,
+                     sample_rate=sample_rate,
+                     frame_rate=frame_rate,
+                     n_fft=512,
+                     center=center))
 
+    # Create train/eval split.
     if eval_split_fraction:
-      examples |= beam.Map(add_key)
+      examples |= beam.Map(_add_key)
       keys = examples | beam.Keys()
-      splits = examples | beam.Partition(eval_split_partition_fn, 2,
+      splits = examples | beam.Partition(_eval_split_partition_fn, 2,
                                          eval_split_fraction,
                                          beam.pvalue.AsList(keys))
 
