@@ -1,4 +1,4 @@
-# Copyright 2021 The DDSP Authors.
+# Copyright 2022 The DDSP Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,7 +21,7 @@ import gin
 import tensorflow as tf
 
 F0_RANGE = ddsp.spectral_ops.F0_RANGE
-LD_RANGE = ddsp.spectral_ops.LD_RANGE
+DB_RANGE = ddsp.spectral_ops.DB_RANGE
 
 tfkl = tf.keras.layers
 
@@ -36,13 +36,13 @@ def at_least_3d(x):
 
 
 def scale_db(db):
-  """Scales [-LD_RANGE, 0] to [0, 1]."""
-  return (db / LD_RANGE) + 1.0
+  """Scales [-DB_RANGE, 0] to [0, 1]."""
+  return (db / DB_RANGE) + 1.0
 
 
 def inv_scale_db(db_scaled):
-  """Scales [0, 1] to [-LD_RANGE, 0]."""
-  return (db_scaled - 1.0) * LD_RANGE
+  """Scales [0, 1] to [-DB_RANGE, 0]."""
+  return (db_scaled - 1.0) * DB_RANGE
 
 
 def scale_f0_hz(f0_hz):
@@ -60,12 +60,27 @@ def inv_scale_f0_hz(f0_scaled):
 class F0LoudnessPreprocessor(nn.DictLayer):
   """Resamples and scales 'f0_hz' and 'loudness_db' features."""
 
-  def __init__(self, time_steps=1000, **kwargs):
+  def __init__(self,
+               time_steps=1000,
+               frame_rate=250,
+               sample_rate=16000,
+               compute_loudness=True,
+               **kwargs):
     super().__init__(**kwargs)
     self.time_steps = time_steps
+    self.frame_rate = frame_rate
+    self.sample_rate = sample_rate
+    self.compute_loudness = compute_loudness
 
-  def call(self, loudness_db, f0_hz) -> [
+  def call(self, loudness_db, f0_hz, audio=None) -> [
       'f0_hz', 'loudness_db', 'f0_scaled', 'ld_scaled']:
+    # Compute loudness fresh (it's fast).
+    if self.compute_loudness:
+      loudness_db = ddsp.spectral_ops.compute_loudness(
+          audio,
+          sample_rate=self.sample_rate,
+          frame_rate=self.frame_rate)
+
     # Resample features to the frame_rate.
     f0_hz = self.resample(f0_hz)
     loudness_db = self.resample(loudness_db)
